/*
 * @(#)CrossDeformCanvas.java   1.0 31/01/2000
 */
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.text.*;

import wc.*;

/**
 * CrossDeformCanvas.java - Shows the deformed structure of Cross Process
 * of continous beams.
 *
 * <p>
 * Description:
 * <p>
 * ===============================================================
 * <p>
 * This file contains an implementation of the Cross Process
 * for continuous beams.  For more details of this process,
 * refer to class notes of course CIV1127-Analise de Estruturas
 * II, Dept. of Civil Engineering, PUC-Rio.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 14/02/2000
 * @author     Luiz Fernando Martha
 * @author     Andre Cahn
 *
 */
class CrossDeformCanvas extends CanvasWC
{
 /** Target main interface panel */
 CrossPanel target;
 
 /** beam line */
 Line2D.Double beam;
 
 /** first structure's support(free rotation) */
 CrossInitFreeRotSup initfreesup;
 
 /** first structure's support(fix rotation) */
 CrossInitFixRotSup initfixsup;
 
 /** internal support */
 CrossIntSup intsup;
 
 /** last structure's support(free rotation) */
 CrossEndFreeRotSup endfreesup;
 
 /** last structure's support(fix rotation) */
 CrossEndFixRotSup endfixsup;
 
 /** node rotation gauge */
 CrossRotationGauge rotation_gauge;
 
 /** Beam total length */
 double len;
 
 /** general path of deform */
 GeneralPath deform;
 
 /** deform factor */
 double deform_fac;
 
 /** stroke object of beam */
 BasicStroke beam_stroke;
 
 /** stroke object of support */
 BasicStroke support_stroke;
 
 /** stroke object of rotation gauge */
 BasicStroke rotgauge_stroke;
 
 /** stroke object of deform */
 BasicStroke deform_stroke;
 
 /** Holds the coordinates of the user's last mousePressed event.*/
 double last_x, last_y;

/*==========================  Constructor  =========================*/
/**
  * Constructor: Sets up vertical Y axis in upward direction.
  * Sets the Deform canvas.
  */
 CrossDeformCanvas( CrossPanel target )
 {
  super( UPWARD_Y );
  this.target = target;
  rotation_gauge  = new CrossRotationGauge(); 
  initfreesup     = new CrossInitFreeRotSup();
  initfixsup      = new CrossInitFixRotSup(); 
  intsup          = new CrossIntSup();        
  endfreesup      = new CrossEndFreeRotSup(); 
  endfixsup       = new CrossEndFixRotSup();  
  len = target.driver.getBeamLength();        
  deform          = new GeneralPath();        
  beam_stroke     = new BasicStroke( (float)(len*0.002) );
  support_stroke  = new BasicStroke( (float)(len*0.001) );
  deform_stroke   = new BasicStroke( (float)(len*0.002) );
  rotgauge_stroke = new BasicStroke( (float)(len*0.001) );
  deform_fac = 1.0;

  setBackground( Color.white );
  setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );

  beam = new Line2D.Double( 0.0, 0.0, len, 0.0 );
 }

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */
 
 /*========================  ComputeDeflection  ======================*/
 /**
  * Compute the deflection of deformed structure. 
  */  
 private double ComputeDeflection( int sup_init, int sup_end, double len, 
                                   double EI, double q, double rotl, 
                                   double rotr, double x )
 {
  double v = 0.0;;

  double a = x/len;
  double a2 = a * a;
  double a3 = a2 * a;
  double a4 = a3 * a;
  double a5 = a4 * a;
  double b = 1.0 - a;
  double b2 = b * b;
  double b3 = b2 * b;
  double b4 = b3 * b;
  double b5 = b4 * b;
  double l2 = len * len;
  double l4 = l2 * l2;
  double displ = 0.0;
  double dispr = 0.0;
  double ql = -q;
  double qr = -q;

  if( (sup_init == 1) && (sup_end == 1) ) 
  {
   v = (1 - 3.0*a2 + 2.0*a3) * displ +
       (a - 2.0*a2 + a3) * len * rotl +
       (3.0*a2 - 2.0*a3) * dispr +
       (-a + a2) * x * rotr +
       (-a2/40.0 + 7.0*a3/120.0 - a4/24.0 + a5/120.0) * (-ql)*l4/EI +
       (-a2/60.0 + a3/40.0 - a5/120.0) * (-qr)*l4 / EI;
  }
  else if( (sup_init == 0) && (sup_end == 1) ) 
  {
   v = (1 - 3.0*a/2.0 + a3/2.0) * displ +
       (3.0*a/2.0 - a3/2.0) * dispr +
       (-1.0/2.0 + a2/2.0) * x * rotr +
       (-a/80.0 + 11.0*a3/240.0 - a4/24.0 + a5/120.0) * (-ql)*l4/EI +
       (-a/120.0 + a3/60.0 - a5/120.0) * (-qr)*l4 / EI;
  }
  else if( (sup_init == 1) && (sup_end == 0) ) 
  {
   v = (1 - 3.0*b/2.0 + b3/2.0) * dispr +
       (3.0*b/2.0 - b3/2.0) * displ + 
       (-1.0/2.0 + b2/2.0) * (len-x) * (-rotl) +
       (-b/80.0 + 11.0*b3/240.0 - b4/24.0 + b5/120.0) * (-qr)*l4/EI +
       (-b/120.0 + b3/60.0 - b5/120.0) * (-ql)*l4 / EI;
  }
  else if( (sup_init == 0) && (sup_end == 0) ) 
  {
   v = (1.0 - a) * displ +
       a * dispr + 
       (-a/45.0 + a3/18.0 - a4/24.0 + a5/120.0) * (-ql)*l4/EI +
       (-7.0*a/360.0 + a3/36.0 - a5/120.0) * (-qr)*l4 / EI;
  }

  return( v );
 }
 
 /*===========================  createDeform  ========================*/
 /**
  * Create the deform line.
  */
 private void createDeform()
 {
  CrossDriver.Member memb;
  double x_beam = 0.0;
  double step = len * 0.02;
  double EI = 10000.0;
  int    n_member = target.driver.member.size();
  int    sup_init;
  int    sup_end;
  int    n_seg;
  double x_member;
  double v_max = 0.0;
  double v;

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   v  = Math.abs( 5.0 * memb.q * (memb.len*memb.len*memb.len*memb.len) / (384.0*EI) );
   if( v > v_max )
    v_max = v;
  }

  deform_fac = (world_area.height * 0.40) / v_max;  

  deform.reset();
  deform.moveTo( (float)x_beam, 0.0f );

  for( int i=0; i<n_member; i++ )
  {
   if( i == 0 )
    sup_init = target.driver.getInitRotationCode();
   else
    sup_init = 1;

   if( i == n_member-1 )
    sup_end = target.driver.getEndRotationCode();
   else
    sup_end = 1;

   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   n_seg = (int)(memb.len/step);
   x_member = 0;
   for( int j=0; j<n_seg-1; j++ )
   {
    x_member += step;
    v = ComputeDeflection( sup_init, sup_end, memb.len, EI, memb.q, 
                           memb.rotl, memb.rotr, x_member );
    deform.lineTo( (float)(x_beam+x_member), (float)(v*deform_fac) );   
   }  
   x_beam += memb.len;
   deform.lineTo( (float)x_beam, 0.0f );     
  }
 }

/*========================= displayRotGauges =========================*/
 /**
  * Display the rotational gauges.
  */
 private void displayRotGauges( Graphics2D g )
 {  
  CrossDriver.Member memb;
  CrossDriver.Member nextmemb;
  double l;    
  double y1;
  double y2;
  double dx;
  double dy;  
  double dxb;
  double dyb;
  double rotarrow_size;
  int    supinit;
  int    supend;
  int    next_supinit;
  int    next_supend;
  double step = len * 0.02;
  double x_beam = 0.0;
  double EI = 10000.0;
  int    n_member = target.driver.member.size();
          
  g.setColor( Color.red );
  g.setStroke( rotgauge_stroke );
  
  rotarrow_size = (world_area.height * 0.2);
  
  for( int i=0; i<n_member-1; i++ )
  {
   if( i == 0 )
    supinit = target.driver.getInitRotationCode();
   else
    supinit = 1;
   supend = 1;

   next_supinit = 1;
   if( i+1 == n_member-1 )
    next_supend = target.driver.getEndRotationCode();
   else
    next_supend = 1;

   memb = (CrossDriver.Member)target.driver.member.elementAt(i);   
   nextmemb = (CrossDriver.Member)target.driver.member.elementAt(i+1);

   if( memb.rotr != 0.0 )
   {
    y1 = ComputeDeflection( supinit, supend, memb.len, EI, memb.q, 
                            memb.rotl, memb.rotr, memb.len - step );  

    y2 = ComputeDeflection( next_supinit, next_supend, nextmemb.len, EI, nextmemb.q, 
                            nextmemb.rotl, nextmemb.rotr, step );

    dx = 2.0 * step;
    dy = - (y2 - y1) * deform_fac;
    l = Math.sqrt( (dx * dx) + ( dy * dy) );
    dxb = dx / l;
    dyb = dy / l;
   }
   else
   {
    dxb = 1.0;
    dyb = 0.0;
   }

   x_beam += memb.len;
   rotation_gauge.draw( g, x_beam, 0.0, -dyb, dxb, rotarrow_size );  
  }    
 } 

 /*=========================   displaySupports =======================*/
 /**
  * Display all supports.
  */
 private void displaySupports( Graphics2D g )
 {
  CrossDriver.Member memb;
  double x = 0.0;
  double x_beam = 0.0;  
  int    n_member = target.driver.member.size();
  
  g.setColor( Color.blue );
  g.setStroke( support_stroke );

  // Display initial support
  if( target.driver.getInitRotationCode() == 0 )
   initfreesup.draw( g, 0.0, 0.0, len * 0.02 );
  else
   initfixsup.draw( g, 0.0, 0.0, len * 0.02 );

  // Display all internal supports
  for( int i=0; i<n_member-1; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x += memb.len;
   intsup.draw( g, x, 0.0, len * 0.02 );
  }

  // Display end support
  if( target.driver.getEndRotationCode() == 0 )
   endfreesup.draw( g, len, 0.0, len * 0.02 );
  else
   endfixsup.draw( g, len, 0.0, len * 0.02 );  
 }

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

/*===============================  reset  ==============================*/
/**
  * Reset method.
  */
 void reset()
 {
  len = target.driver.getBeamLength();
  beam_stroke = new BasicStroke( (float)(len*0.002) );
  support_stroke = new BasicStroke( (float)(len*0.001) );
  deform_stroke = new BasicStroke( (float)(len*0.002) );
  setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );
  beam.setLine( 0.0, 0.0, len, 0.0 );
  repaint();
 }

/*===============================  redraw  =============================*/
 /**
  * Redisplay method.
  * This method is called every time the canvas is updated.
  * Client's concrete class must implement it.
  * @param g is the 2D graphics context that the client uses to display
  */
 public void redraw( Graphics2D g )
 {
  g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                      RenderingHints.VALUE_ANTIALIAS_ON );
	
  // Update beam deformed configuration
  createDeform();

  // Display beam line
  g.setColor( Color.blue );
  g.setStroke( beam_stroke );
  g.draw( beam );

  // Display all supports
  displaySupports( g );
  
  // Display deformed configuration
  g.setStroke( deform_stroke );
  g.draw( deform );

  // Display rotation gauges
  displayRotGauges( g );
 }

/**
 * This class defines the rotation gauge figure.
 */
 class CrossRotationGauge
 {      
  Arc2D.Double arc;       // point of the rotational scale effect
  Point2D.Double up1;     // point of the rotational scale effect
  Point2D.Double down1;   // point of the rotational scale effect
  Point2D.Double up2;     // point of the rotational scale effect
  Point2D.Double down2;   // point of the rotational scale effect  
  Line2D.Double scaleln;  // line of the rotational scale effect
  Point2D.Double toppt;   // point of the rotational arrow
  Point2D.Double botpt;   // point of the rotational arrow
  Point2D.Double leftpt;  // point of the rotational arrow
  Point2D.Double rightpt; // point of the rotational arrow
  Line2D.Double arrowln;  // line of the rotational arrow
  Line2D.Double leftln;   // line of the rotational arrow
  Line2D.Double rightln;  // line of the rotational arrow
  
/*==========================  Constructor  =========================*/
  CrossRotationGauge()
  {   
   arc     = new Arc2D.Double();  
   scaleln = new Line2D.Double();
   botpt   = new Point2D.Double(  0.0,  0.0 );   
   toppt   = new Point2D.Double(  0.0,  1.0 );
   leftpt  = new Point2D.Double( -0.1,  0.8 );
   rightpt = new Point2D.Double(  0.1,  0.8 );
   arrowln = new Line2D.Double( toppt, botpt );
   leftln  = new Line2D.Double( leftpt, toppt );
   rightln = new Line2D.Double( rightpt, toppt );   
  }

/*=============================  draw  =============================*/
/**
 * Draw the rotation gauge.
 */
  void draw( Graphics2D g, double x,  double y, double dxb, double dyb, double size )
  {
   double a = 0.0;         /** Angle beetween the traces of scale effects */
   double xx = 0.0;
   double yy = 0.0;

   // Display the rotational scale traces effect   
   a = 45.0;
   for( int i=0; i<7; i++ )
   {
    // raio = 1.25*size
    xx = 1.25*size * Math.cos( Math.toRadians(a) );
    yy = 1.25*size * Math.sin( Math.toRadians(a) );
    scaleln.x1 = x + xx;
    scaleln.y1 = y + yy;
    scaleln.x2 = x + 0.85*xx;
    scaleln.y2 = y + 0.85*yy;
    g.draw( scaleln );
    a += 15.0;
   }
   a = 52.5;
   for( int j=0; j<6; j++ )
   {    
    // raio = 1.25*size
    xx = 1.25*size * Math.cos( Math.toRadians(a) );
    yy = 1.25*size * Math.sin( Math.toRadians(a) );
    scaleln.x1 = x + xx;
    scaleln.y1 = y + yy;
    scaleln.x2 = x + 0.90*xx;
    scaleln.y2 = y + 0.90*yy;
    g.draw( scaleln );
    a += 15.0;   
   }

   // Display the rotational scale arc
   arc.setArc( x-(1.25*size), y-(1.25*size), size*2.5, size*2.5, 225.0, 90.0, Arc2D.OPEN );
   
   // Display the rotational arrow
   double tx = -x*dyb + y*dxb + x;
   double ty = -x*dxb - y*dyb + y;
      
   arrowln.x1 = x;
   arrowln.y1 = y;
   arrowln.x2 = (x + toppt.getX()*size)*dyb - (y + toppt.getY()*size)*dxb + tx;
   arrowln.y2 = (x + toppt.getX()*size)*dxb + (y + toppt.getY()*size)*dyb + ty;

   leftln.x1 = (x + leftpt.getX()*size)*dyb - (y + leftpt.getY()*size)*dxb + tx;
   leftln.y1 = (x + leftpt.getX()*size)*dxb + (y + leftpt.getY()*size)*dyb + ty;
   leftln.x2 = arrowln.x2;
   leftln.y2 = arrowln.y2;

   rightln.x1 = (x + rightpt.getX()*size)*dyb - (y + rightpt.getY()*size)*dxb + tx;
   rightln.y1 = (x + rightpt.getX()*size)*dxb + (y + rightpt.getY()*size)*dyb + ty;
   rightln.x2 = arrowln.x2;
   rightln.y2 = arrowln.y2;

   g.draw( arrowln );
   g.draw( leftln );
   g.draw( rightln );    
   g.draw( arc );
  }
 } 

 /**
  * Handles the event of the user pressing down the mouse button.
  */
 public void mousePressedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  last_x = mouse_pos.x;
  last_y = mouse_pos.y;
 }

 /**
  * Handles the event of a user dragging the mouse while holding down the mouse button.
  */
 public void mouseDraggedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }

 /**
  * Handles the event of a user releasing the mouse button.
  */
  public void mouseReleasedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  target.controls.resetButtons();
 }
}