/*
 * @(#)CrossDriver.java   1.0 31/01/2000
 */

import java.util.Vector;

/**
 * CrossDriver.java - Driver of Cross Process of continous beams.
 *
 * <p>
 * Description:
 * <p>
 * ===============================================================
 * <p>
 * This file contains driver routines for solving moments of 
 * continuous beams by the Cross Process.  For more details of 
 * this process, refer to class notes of course CIV1127-Analise 
 * de Estruturas II, Dept. of Civil Engineering, PUC-Rio.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 31/01/2000
 * @author     Luiz Fernando Martha
 * @author     Andre Cahn
 *
 */
class CrossDriver
{

 /*
 ** ---------------------------------------------------------------
 ** Local classes: 
 */

 /**
  * This class defines a member of a continuous beam for the Cross applet.
  */
 class Member
 {
  double len;       /* length */
  double q;         /* vertical distributed load (top-down positive) */
  double ml;        /* left end moment */
  double mr;        /* right end moment */
  double rotl;      /* left end rotation */
  double rotr;      /* right end rotation */
 }


 /*
 ** ---------------------------------------------------------------
 ** Fields:
 */
 /** Constant that defines the maximum number of members */
 static final int MAX_NUM_MEMBER = 50;

 /** Auxiliary array of member length */
 double len[];

 /** Auxiliary array of member load */
 double q[];

  /** Auxiliary array of node rotation */
 double rot[];

 /** Auxiliary array of left member moment */
 double ml[];

 /** Auxiliary array of right member moment */
 double mr[];

 /** Target main interface panel */
 CrossPanel target;

 /** Solver of Cross Process */
 CrossSolver solver;

 /** Number of members. */
 int n_member;

 /** Left end support condition. */
 int sup_init = 0;

 /** Right end support condition. */
 int sup_end = 0;

 /** Moment precision (number of decimal places) */
 int mom_prec = 1;

 /** Tolerance for moment balancing. */
 double mom_tol = 0.1;

 /** Main vector for members. */
 Vector member;

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */

 /*==========================  createSolver  =======================*/
 /**
  * Create a CrossSolver object from current data.
  */
 protected void createSolver()
 {
  Member memb;
  n_member = member.size();
  for( int i=0; i<n_member; i++ )
  {
   memb = (Member)member.elementAt(i);
   len[i] = memb.len;
   q[i] = memb.q;
  }
  solver = new CrossSolver( n_member, sup_init, sup_end, mom_tol, len, q );
 }

 /*==========================  updateMoments  ========================*/
 /**
  * Get current member moments from Cross solver.
  */
 protected void updateMoments()
 {
  Member memb;
  solver.getMoments( ml, mr );
  for( int i=0; i<member.size(); i++ )
  {
   memb = (Member)member.elementAt(i);
   memb.ml = ml[i];
   memb.mr = mr[i];
  }
 }

 /*=========================  updateRotations  =======================*/
 /**
  * Get current node rotations from Cross solver.
  */
 protected void updateRotations()
 {
  Member membl;
  Member membr;

  solver.getRotations( rot );

  membl = (Member)member.elementAt(0);
  membl.rotl = 0.0;
  for( int i=0; i<member.size()-1; i++ )
  {
   membl = (Member)member.elementAt(i);
   membr = (Member)member.elementAt(i+1);
   membl.rotr = rot[i];
   membr.rotl = rot[i];
  }
  membr = (Member)member.elementAt(member.size()-1);
  membr.rotr = 0.0;
 }

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */
 /*=========================  Constructor  ==========================*/

 CrossDriver( CrossPanel target )
 {
  this.target = target;

  len = new double[MAX_NUM_MEMBER];
  q = new double[MAX_NUM_MEMBER];
  rot = new double[MAX_NUM_MEMBER-1];
  ml = new double[MAX_NUM_MEMBER];
  mr = new double[MAX_NUM_MEMBER];

  mom_prec = 1;
  mom_tol = 0.1;

  sup_init = 0;
  sup_end = 1;

  n_member = 3;

  len[0] = 8.0;
  len[1] = 6.0;
  len[2] = 8.0;
  q[0] = 8.0;
  q[1] = 6.0;
  q[2] = 9.0;

  createMemberVector();
  resetCrossSolver();
 }

 /*=======================  createMemberVector  =========================*/

 void createMemberVector()
 {
  Member memb;

  if( member == null )
   member = new Vector();
  else
   member.removeAllElements();

  for( int i=0; i<n_member; i++ )
  {
   memb = new Member();
   memb.len = len[i];
   memb.q = q[i];
   member.add( memb );
  }
 }

 /*=========================  resetCrossSolver  =========================*/

 void resetCrossSolver()
 {
  createSolver();
  solver.resetSolver();
  updateMoments();
  updateRotations();
 }

 /*===========================  delIntSupport  ==========================*/

 boolean delIntSupport( int n )
 {
  if( member.size() < 3 )
   return( false );

  if( (n < 0) || (n > member.size()-2) )
   return( false );

  Member memb = (Member)member.elementAt(n);
  Member memb_next = (Member)member.elementAt(n+1);
  double next_len = memb_next.len;

  member.removeElementAt(n+1);
  memb.len += next_len;

  resetCrossSolver();

  return( true );
 }

 /*===========================  addIntSupport  ==========================*/

 boolean addIntSupport( int m, double x )
 {
  Member memb;
  Member newmemb;
  double x_left = 0.0;

  for( int i=0; i<m; i++ )
  {
   memb = (Member)member.elementAt(i);
   x_left += memb.len;
  }

  memb = (Member)member.elementAt(m);

  if( (x <= x_left) || (x >= x_left+memb.len) )
   return( false );

  newmemb = new Member();
  newmemb.len = x - x_left;
  newmemb.q = memb.q;
  memb.len -= newmemb.len;

  member.insertElementAt( newmemb, m );

  resetCrossSolver();

  return( true );
 }

 /*==========================  moreCrossSteps  ==========================*/

 boolean moreCrossSteps()
 {
  return( solver.moreStepsToGo() );
 }

 /*==========================  autoStepCross  ===========================*/

 boolean autoStepCross()
 {
  if( solver.autoStepSolver() )
  {
   updateMoments();
   updateRotations();
   return( true );
  }
  return( false );
 }

 /*==========================  userStepCross  ===========================*/

 boolean userStepCross( int node )
 {
  if( solver.nodeStepSolver( node ) )
  {
   updateMoments();
   updateRotations();
   return( true );
  }
  return( false );
 }

 /*===========================  goThruCross  ============================*/

 void goThruCross()
 {
  solver.goThruSolver();
  updateMoments();
  updateRotations();
 }

 /*=========================  fixInitRotation  ==========================*/

 void fixInitRotation()
 {
  if( sup_init != 1 )
  {
   sup_init = 1;
   resetCrossSolver();
  }
 }

 /*=======================  releaseInitRotation  ========================*/

 void releaseInitRotation()
 {
  if( sup_init != 0 )
  {
   sup_init = 0;
   resetCrossSolver();
  }
 }

 /*=======================  getInitRotationCode  ========================*/

 int getInitRotationCode()
 {
  return( sup_init );
 }

 /*==========================  fixEndRotation  ==========================*/

 void fixEndRotation()
 {
  if( sup_end != 1 )
  {
   sup_end = 1;
   resetCrossSolver();
  }
 }

 /*========================  releaseEndRotation  ========================*/

 void releaseEndRotation()
 {
  if( sup_end != 0 )
  {
   sup_end = 0;
   resetCrossSolver();
  }
 }

 /*=======================  getEndRotationCode  =========================*/

 int getEndRotationCode()
 {
  return( sup_end );
 }

 /*========================  setMomentPrecision  ========================*/

 void setMomentPrecision( int mom_prec )
 {
  switch( mom_prec )
  {
   case 0:
    this.mom_prec = 0;
    mom_tol = 1.0;
    break;
   default:
   case 1:
    this.mom_prec = 1;
    mom_tol = 0.1;
    break;
   case 2:
    this.mom_prec = 2;
    mom_tol = 0.01;
    break;
  }
  solver.setMomentToler( mom_tol );
 }

 /*========================  getMomentPrecision  ========================*/

 int getMomentPrecision()
 {
  return( mom_prec );
 }

 /*===========================  getBeamLength  ==========================*/

 double getBeamLength()
 {
  Member memb;
  double len = 0.0;
  for( int i=0; i<member.size(); i++ )
  {
   memb = (Member)member.elementAt(i);
   len += memb.len;
  }
  return( len );
 }
}