/*
 * @(#)CrossDriverIO.java   1.0 05/06/2000
 */

import java.awt.Component;
import java.io.*;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

/**
 * CrossDriverIO.java - IO Driver of Cross Process of continous beams.
 *
 * <p>
 * Description:
 * <p>
 * ===============================================================
 * <p>
 * This file contains IO driver routines for solving moments of 
 * continuous beams by the Cross Process.  For more details of 
 * this process, refer to class notes of course CIV1127-Analise 
 * de Estruturas II, Dept. of Civil Engineering, PUC-Rio.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 05/06/2000
 * @author     Luiz Fernando Martha
 * @author     Andre Cahn
 *
 */
class CrossDriverIO
{
 /*
 ** ---------------------------------------------------------------
 ** Fields:
 */
 /** Target main interface panel */
 CrossPanel target;

 JFileChooser f_chooser;

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */
 /*=========================  Constructor  ==========================*/

 CrossDriverIO( CrossPanel target )
 {
  this.target = target;
  f_chooser = new JFileChooser(); 
  f_chooser.setCurrentDirectory( new File(".") );
  CrossFileFilter f_filter = new CrossFileFilter( "crs", "Cross Files" );
  f_chooser.addChoosableFileFilter( f_filter);
  f_chooser.setMultiSelectionEnabled( false );
 }

 /*===========================  openInputCrossFile  ==========================*/

 boolean openInputCrossFile( Component parent )
 {
  target.model_canvas.support_insertion = false;
  target.controls.support.setIcon( target.controls.support1ButtonIcon );
  target.model_canvas.support_deletion = false;
  target.controls.delete.setIcon( target.controls.delete1ButtonIcon );
  f_chooser.setDialogTitle( "Open existing Cross model file." );
  if( f_chooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION  )
  {
   if( ! readCrossFile(f_chooser.getSelectedFile()) )
   {
    target.setMsg( "Problems reading Cross model file." );
    return( false );
   }
   else
   {
    target.setMsg( "Try to do something." );
    target.controls.resetButtons();
    target.model_canvas.reset();
    target.deform_canvas.reset();
    target.moment_canvas.reset();
    target.solution_canvas.reset();
   }
  }
  else
  {
   return( false );
  }
  return( true );
 }

 /*===========================  readCrossFile  ==========================*/

 protected boolean readCrossFile( File cross_file )
 {
  try
  {
   BufferedReader in = new BufferedReader( new FileReader( cross_file ) );
   String line;
   String first;
   String second;
   int    idx;
   int    mom_prec;
   int    n_member;
   int    sup_init;
   int    sup_end;

   line = in.readLine();
   if( line.length() == 0 ) return( false );
   mom_prec = Integer.parseInt( line );
   if( (mom_prec < 0) || (mom_prec > 2) )
    return( false );

   line = in.readLine();
   if( line.length() == 0 ) return( false );
   idx = line.indexOf( " " );
   first = line.substring( 0, idx );
   if( first.length() == 0 ) return( false );
   sup_init = Integer.parseInt( first );
   if( sup_init != 0 )
    sup_init = 1;
   idx = line.lastIndexOf( " " );
   second = line.substring( idx+1 );
   if( second.length() == 0 ) return( false );
   sup_end = Integer.parseInt( second );
   if( sup_end != 0 )
    sup_end = 1;

   line = in.readLine();
   if( line.length() == 0 ) return( false );
   n_member = Integer.parseInt( line );
   if( n_member < 2 )
    return( false );

   if( n_member > target.driver.MAX_NUM_MEMBER )
    return( false );

   for( int i=0; i<n_member; i++ )
   {
    line = in.readLine();
    if( line.length() == 0 ) return( false );
    idx = line.indexOf( " " );
    first = line.substring( 0, idx );
    if( first.length() == 0 ) return( false );
    target.driver.len[i] = Double.parseDouble( first );
    idx = line.lastIndexOf( " " );
    second = line.substring( idx+1 );
    if( second.length() == 0 ) return( false );
    target.driver.q[i] = Double.parseDouble( second );
   }

   in.close();

   target.driver.setMomentPrecision( mom_prec );
   target.driver.sup_init = sup_init;
   target.driver.sup_end = sup_end;
   target.driver.n_member = n_member;
   target.driver.createMemberVector();
   target.driver.resetCrossSolver();
  }
  catch( Exception e )
  {
   return( false );
  }
  return( true );
 }

 /*===========================  openOutputCrossFile  ==========================*/

 boolean openOutputCrossFile( Component parent )
 {
  target.model_canvas.support_insertion = false;
  target.controls.support.setIcon( target.controls.support1ButtonIcon );
  target.model_canvas.support_deletion = false;
  target.controls.delete.setIcon( target.controls.delete1ButtonIcon );
  f_chooser.setDialogTitle( "Save current Cross model in a file." );
  if( f_chooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION  )
  {
   if( ! writeCrossFile(f_chooser.getSelectedFile()) )
   {
    target.setMsg( "Problems writing Cross model file." );
    return( false );
   }
   else
   {
    target.setMsg( "Try to do something." );
   }
  }
  else
  {
   return( false );
  }
  return( true );
 }

 /*===========================  writeCrossFile  =========================*/

 protected boolean writeCrossFile( File cross_file )
 {
  try
  {
   BufferedWriter out = new BufferedWriter( new FileWriter( cross_file ) );
   CrossDriver.Member memb;

   out.write( Integer.toString( target.driver.mom_prec ) );
   out.newLine();

   out.write( Integer.toString( target.driver.sup_init ) + "  " + Integer.toString( target.driver.sup_end ) );
   out.newLine();

   out.write( Integer.toString( target.driver.member.size() ) );
   out.newLine();

   for( int i=0; i<target.driver.member.size(); i++ )
   {
    memb = (CrossDriver.Member)target.driver.member.elementAt(i);
    out.write( Double.toString( memb.len ) + "  " + Double.toString( memb.q ) );
    out.newLine();
   }

   out.close();
   out.flush();
  }
  catch( Exception e )
  {
   return( false );
  }
  return( true );
 }
}

/**
 * CrossFileFilter - Auxiliary class to filter cross model files.
 */
class CrossFileFilter extends FileFilter
{
 private String m_description = null;
 private String m_extension = null;

 public CrossFileFilter(String extension, String description) 
 {
  m_description = description;
  m_extension = "."+extension.toLowerCase();
 }

 public String getDescription()
 {
  return m_description;
 }

 public boolean accept(File f) 
 {
  if (f == null) 
   return false;
  if (f.isDirectory())
   return true;
  return f.getName().toLowerCase().endsWith(m_extension);
 }
}