/*
 * @(#)CrossModelCanvas.java   1.0 31/01/2000
 */
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.text.*;
import java.util.Vector;

import wc.*;

/**
 * CrossModelCanvas.java - Shows the loaded structure of Cross Process
 * of continous beams.
 *
 * <p>
 * Description: 
 * <p>
 * ===============================================================
 * <p>
 * This file contains an implementation of the Cross Process
 * for continuous beams.  For more details of this process,
 * refer to class notes of course CIV1127-Analise de Estruturas
 * II, Dept. of Civil Engineering, PUC-Rio.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 31/01/2000
 * @author     Luiz Fernando Martha
 * @author     Andre Cahn
 *
 */
class CrossModelCanvas extends CanvasWC
{
 CrossPanel target;
 boolean support_insertion = false; /** flag to insertion of a support */
 boolean support_deletion = false;  /** flag to deletion of a support  */
 Line2D.Double beam;                /** beam line  */
 double len;                        /** length of beam */
 double x_init;                     /** initial beam x coordinate */
 double x_end;                      /** end beam x coordinate */
 double min_l;                      /** minimum member length */
 double load_fac = 0.0;             /** scaling factor for displaying loads */
 Line2D.Double load_line;           /** the top line of the loading effect */
 Line2D.Double rule_line;           /** bar to show the dimensions of the beams */ 
 CrossRuleSB rulesb;                /** effects to show the end and begin of rules */
 CrossLoadArrow load_arrow;         /** arrows that represents the load  */
 CrossInitFreeRotSup initfreesup;   /** first structure's support(free rotation) */
 CrossInitFixRotSup initfixsup;     /** first structure's support(fix rotation) */
 CrossIntSup intsup;                /** internal support */
 CrossEndFreeRotSup endfreesup;     /** last structure's support(free rotation) */
 CrossEndFixRotSup endfixsup;       /** last structure's support(fix rotation) */
 BasicStroke beam_stroke;           /** stroke object of beam */
 BasicStroke support_stroke;        /** stroke object of support */
 BasicStroke load_stroke;           /** stroke object of the top line of the loading */
 BasicStroke rule_stroke;           /** stroke object of the dimensions lines */
 int pickedNode;                    /** index of a picked node (from 0 to n_member) */
 CrossDriver.Member pickedMemb;     /** member found in a load pick */
 DecimalFormat form_load;           /** load values number format object */
 DecimalFormat form_len;            /** length values number format object */

 /** Holds the coordinates of the user's last mouse event. */
 double last_x, last_y;

 /** True if the user pressed, dragged or released the mouse at 
  *  an internal support; false otherwise. */
 boolean pressedInNode;  

 /** True if the user pressed, dragged or released the mouse at 
  *  a load top line; false otherwise. */
 boolean pressedInLoad;  

 /*=========================  Constructor  ==========================*/

 /**
  * Constructor:  Sets up vertical Y axis in upward direction.
  * Sets the Model Canvas.
  */
 CrossModelCanvas( CrossPanel target )
 {
  super( UPWARD_Y );
  this.target = target;
  support_insertion = false;
  support_deletion = false;
  initfreesup = new CrossInitFreeRotSup();
  initfixsup  = new CrossInitFixRotSup();
  intsup      = new CrossIntSup();
  endfreesup  = new CrossEndFreeRotSup();
  endfixsup   = new CrossEndFixRotSup();
  len = target.driver.getBeamLength();
  x_init = 0.0;
  x_end = len;
  min_l = len / 10.0;
  beam_stroke    = new BasicStroke( (float)(len*0.002) );
  support_stroke = new BasicStroke( (float)(len*0.001) );
  load_stroke    = new BasicStroke( (float)(len*0.001) );
  rule_stroke    = new BasicStroke( (float)(len*0.001) );

  setFont( new Font( "helvetica", Font.PLAIN, 10 ) );
  
  setBackground( Color.white );
  setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );
  
  beam       = new Line2D.Double( 0.0, 0.0, len, 0.0 );
  load_line  = new Line2D.Double( 0.0, 1.0, 1.0, 1.0 );
  load_arrow = new CrossLoadArrow();
  rulesb     = new CrossRuleSB();
  rule_line  = new Line2D.Double( 0.0, -1.0, len, -1.0);

  pressedInNode = false;
  pressedInLoad = false;

  try {
   form_load = new DecimalFormat( "####0" );
   form_len = new DecimalFormat( "####0.0" );
  } 
  catch( IllegalArgumentException e ) {
  }
 }

 /**
  *  This private method returns the index of a support that 
  *  is closed to the given point.  The index ranges from 0
  *  (left support) to the number of beam members (right support).
  */
 private int pickNode( Point2D.Double pt )
 {
  CrossDriver.Member memb;
  double tol_fac = 0.01;
  double tol = len * tol_fac;
  int    n_member = target.driver.member.size();
  double x_beam = x_init;
  double y_beam = 0.0;
  double x_member;

  if( (Math.abs(pt.x - x_beam) <= tol) &&
      (Math.abs(pt.y - y_beam) <= tol) )
  {
   return( 0 );
  }

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x_beam += memb.len;

   if( (Math.abs(pt.x - x_beam) <= tol) &&
       (Math.abs(pt.y - y_beam) <= tol) )
   {
   	// The position of node 2 is x_beam = elementAt(1), 
   	// because x_beam is the end of member1. So return(i+1).
    return( i+1 );
   }
  }

  return( -1 );
 }

/**
  *  This  private  method returns the  index of a member that 
  *  is closed to the given point. A invalid index is returned 
  *  if the distance between the given point and the load line
  *  is bigger than the specified  tolerance. The index ranges 
  *  from  0  (first  mamber)  to  the  number of beam members 
  *  (last member).
  */
 private int pickLoad( Point2D.Double pt )
 {
  CrossDriver.Member memb;
  double tol_fac = 0.005;
  double tol = 5.0*len * tol_fac;
  int    n_member = target.driver.member.size();
  double x_left = x_init;
  double x_right;
  double y_load;

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x_right = x_left + memb.len;

   y_load = memb.q * load_fac;

   if( (pt.x > (x_left  + tol)) &&
       (pt.x < (x_right - tol)) && 
       (Math.abs(pt.y - y_load) <= tol) )
   {
    return( i );
   }
   x_left = x_right;
  }
  return( -1 );
 }

 /**
  *  This  private method returns the index of a member where a
  *  new internal support will be created. The index ranges from 0
  *  (first member) to the number of beam members - 1 (last member).
  *  The internal support will not be created if one of the
  *  members resulting from the division of the found member    
  *  would have length smaller than the specified minimum length.
  *  If the given point does not satisfy these conditions, an 
  *  invalid index (-1) is returned.
  *  If the given point is outside the beam limits, an invalid
  *  index (-2) is returned.
  */
 private int getIntSupMember( Point2D.Double pt )
 {
  CrossDriver.Member memb;
  double tol_fac = 0.015;
  double tol = len * tol_fac;
  int    n_member = target.driver.member.size();
  double x_left = x_init;
  double x_right;
  double y_beam = 0.0;

  if( pt.x < x_left )
   return( -2 );

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x_right = x_left + memb.len;

   if( (pt.x - x_left > min_l) && (x_right - pt.x > min_l) && 
       (Math.abs(pt.y - y_beam) <= tol) )
   {
    return( i );
   }
   x_left = x_right;
  }

  if( pt.x > x_left )
   return( -2 );

  return( -1 );
 }
 
 /*========================= displaySupports ==========================*/
 /**
  * Display all supports.
  */	
 private void displaySupports( Graphics2D g )
 {
  CrossDriver.Member memb;
  double  x = x_init;
  int     n_member = target.driver.member.size();

  g.setColor( Color.blue );
  g.setStroke( support_stroke );

  // Display initial support
  if( target.driver.getInitRotationCode() == 0 )
   initfreesup.draw( g, x_init, 0.0, len * 0.02 );
  else
   initfixsup.draw( g, x_init, 0.0, len * 0.02 );

  // Display all internal supports
  for( int i=0; i<n_member-1; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x += memb.len;
   intsup.draw( g, x, 0.0, len * 0.02 );
  }

  // Display end support
  if( target.driver.getEndRotationCode() == 0 )
   endfreesup.draw( g, x_end, 0.0, len * 0.02 );
  else
   endfixsup.draw( g, x_end, 0.0, len * 0.02 );
 }

/*=========================== displayLoads ===========================*/
 /**
  * Display load lines, arrows and values.
  */
 private void displayLoads( Graphics2D g )
 {
  String val;
  CrossDriver.Member memb;
  double step_fac = 0.02;
  double step = len * step_fac;
  int    n_member = target.driver.member.size();
  int    n_seg;
  double x_beam = x_init;
  double x_member;
 
  g.setColor( Color.black );
  g.setStroke( load_stroke );

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);

   // Display load line
   load_line.setLine( x_beam, memb.q*load_fac, 
                      x_beam + memb.len, memb.q*load_fac );
   g.draw( load_line );
 
   // Display load arrows
   n_seg = (int)(memb.len/step);
   if( n_seg == 0 )
    n_seg = 1;
   step = memb.len / n_seg;
   x_member = x_beam;
   for( int j=0; j<n_seg+1; j++ )
   {
    if( memb.q > 0.0 )
     load_arrow.draw( g, x_member, 0.0, memb.q*load_fac, step );
    else if( memb.q < 0.0 )
     load_arrow.draw( g, x_member, 0.0, memb.q*load_fac, -step );
    x_member += step;
   }

   // Display load value
   g.setColor( Color.black );
   val = new String( form_load.format(Math.abs(memb.q)) + " kN/m" );
   if( memb.q >= 0.0 )
   {
    setTextAlignment( CanvasWC.BASE_CENTER );
    drawString( val, (x_beam + (memb.len*0.5)), (memb.q*load_fac + len*0.002) );
   }
   else
   {
    setTextAlignment( CanvasWC.NORTH );
    drawString( val, x_beam + (memb.len*0.5), memb.q*load_fac - len*0.002 );
   }

   x_beam += memb.len;
   step = len * step_fac;
  }
 }

 /**
  *  This private method displays the length values of members,
  *  and a scale line.
  */
 private void displayRule( Graphics2D g )
 {
  String val;
  CrossDriver.Member memb;
  double valpos;
  int    n_member = target.driver.member.size();
  double x_beam = x_init;
  double x_member;
  double lastlen;
   
  g.setColor( Color.gray);
  g.setStroke( load_stroke );
  
  valpos = world_area.height * -0.42;
  
  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x_member = memb.len;
   
   // Display beam size line
   rule_line.setLine( x_beam, valpos, 
                      x_beam + memb.len, valpos );
   g.draw( rule_line );
   
   // Display beam length value
   val = new String( form_len.format(Math.abs(memb.len)) + " m" );
   setTextAlignment( CanvasWC.CENTER );
   setTextOpacity( CanvasWC.OPAQUE );
   drawString( val, (x_beam + (memb.len*0.5)), valpos );
   setTextOpacity( CanvasWC.TRANSPARENT );

   // Display side bars of the size line 
   rulesb.draw( g, x_beam, valpos, 0.2*valpos );
   if(i == n_member - 1)
    rulesb.draw( g, x_beam + memb.len, valpos, 0.2*valpos );
      
   x_beam += memb.len; 
  }
 }

/** Compute maximum load value and set load display factor
 *  according to this value such that maximum load display 
 *  does not take more than 1/4 of canvas height.
 */
 private void computeLoadFactor()
 {
  double max_q = 0.0;
  double q;
  CrossDriver.Member memb;
  for( int i=0; i<target.driver.member.size(); i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   q  = Math.abs( memb.q );
   if( q > max_q )
    max_q = q;
  }
  if( max_q == 0.0 )
   max_q = 10.0;

  load_fac = (world_area.height * 0.25) / max_q;
 }

/*===============================  reset  ==============================*/
/**
  * Reset method.
  */
 void reset()
 {
  len = target.driver.getBeamLength();
  x_init = 0.0;
  x_end = len;
  min_l = len / 10.0;
  beam_stroke    = new BasicStroke( (float)(len*0.002) );
  support_stroke = new BasicStroke( (float)(len*0.001) );
  load_stroke    = new BasicStroke( (float)(len*0.001) );
  rule_stroke    = new BasicStroke( (float)(len*0.001) );
  setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );
  beam.setLine( x_init, 0.0, x_end, 0.0 );

  computeLoadFactor();
  repaint();
 }

/*===============================  redraw  =============================*/
 /**
  * Redisplay method.
  * This method is called every time the canvas is updated.
  * Client's concrete class must implement it.
  * @param g is the 2D graphics context that the client uses to display
  */
 public void redraw( Graphics2D g )
 {
  if( load_fac == 0.0 )
  {
   reset();
   return;
  }
  else
  {
   if( !pressedInNode && !pressedInLoad )
    computeLoadFactor();
  }

  g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
  	              RenderingHints.VALUE_ANTIALIAS_ON );

  // Display beam line
  g.setColor( Color.blue );
  g.setStroke( beam_stroke );
  beam.setLine( x_init, 0.0, x_end, 0.0 );
  g.draw( beam );

  displaySupports( g );

  displayLoads( g );
  
  displayRule( g );
 }

/*============================  updateNodePos ==========================*/
/**
 * This private methods tries to change the position of a node
 * as a result of a mouse dragging event.
 */
 private void updateNodePos( int n, Point2D.Double mouse_pos )
 {
  double x_var = mouse_pos.x - last_x;
  int    n_member = target.driver.member.size();
  CrossDriver.Member memb_left;    /* member at left of node */
  CrossDriver.Member memb_right;   /* member at right of node */
  double x;                        /* holds current node position */
  double x_new;                    /* holds new node position */
  double x_min;                    /* minimum value for node position */
  double x_max;                    /* maximum value for node position */
  int    ip;		           /* integer part */
  double fp;		           /* "fraction" part */
  double prec = 0.1;               /* precision value */

  if( n == 0 )
  {
   // Handle left end node
   x = x_init;
   memb_left  = null;
   memb_right = (CrossDriver.Member)target.driver.member.elementAt(0);
   x_min = world_area.x;
   x_max = x_init + memb_right.len - min_l;
  }
  else if( n == n_member )
  {
   // Handle right end node
   x = x_end;
   memb_left  = (CrossDriver.Member)target.driver.member.elementAt(n_member-1);
   memb_right = null;
   x_min = x_end - memb_left.len + min_l;
   x_max = world_area.x + world_area.width;
  }
  else
  {
   // Handle internal node
   x = x_init;
   for( int i=0; i<n; i++ )
   {
    memb_right = (CrossDriver.Member)target.driver.member.elementAt(i);
    x += memb_right.len;
   }
   memb_left  = (CrossDriver.Member)target.driver.member.elementAt(n-1);
   memb_right = (CrossDriver.Member)target.driver.member.elementAt(n);
   x_min = x - memb_left.len  + min_l;
   x_max = x + memb_right.len - min_l;
  }

  // Snap new node position value to the closest value with one decimal place.
  x_new = x + x_var;
  fp = x_new / prec;
  ip = (int)fp;
  fp = fp - ip;
  if( fp > 0.5 )
   x_new = (ip + 1.0) * prec;
  else if( fp < -0.5 )
   x_new = (ip - 1.0) * prec;
  else
   x_new = ip * prec;

  // Do not allow shifting the node outside the limit range. 
  if( (x_new < x_min) || (x_new > x_max) )
   return;

  // Update lengths of members adjacent to the moved node and redisplay.
  x_var = x_new - x;

  if( n == 0 )
   x_init = x_new;

  if( n == n_member )
   x_end = x_new;

  if( memb_left != null )
   memb_left.len += x_var;
  if( memb_right != null )
   memb_right.len -= x_var;

  repaint();
 }

/*===========================  updateLoadValue =========================*/
/**
 * This private methods tries to change the value of a member
 * load as a result of a mouse dragging event.
 */
 private void updateLoadValue( CrossDriver.Member memb, Point2D.Double mouse_pos )
 {
  double y_var = mouse_pos.y - last_y;
  double q_var = y_var / load_fac;
  double q_new = memb.q + q_var;
  int    ip;		/* integer part */
  double fp;		/* "fraction" part */
  double prec = 1.0;    /* precision value */

  // Snap new load value to the closest value with no decimal place.
  fp = q_new / prec;
  ip = (int)fp;
  fp = fp - ip;
  if( fp > 0.5 )
   q_new = (ip + 1.0) * prec;
  else if( fp < -0.5 )
   q_new = (ip - 1.0) * prec;
  else
   q_new = ip * prec;

  // Do not allow shifting the load top line up to the 
  // canvas vertical limits.
  if( Math.abs(q_new * load_fac) > (world_area.height * 0.40) )
   return;

  // Update member load value and redisplay.
  memb.q = q_new;
  repaint();
 }

/*==============================  mouseMoved ===========================*/
 /**
  * Handles the event of mouse motion in canvas with no
  * button pressed.
  */
 public void mouseMoved( MouseEvent e )
 {
  if( raster_area == null )
   return;

  if( !support_insertion && !support_deletion )
  {
   Point2D.Double mouse_pos = raster2world( e.getX(), e.getY() );
   int n = pickNode( mouse_pos );
   if( n > -1 )
   {
    setCursor( Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR) );
   }
   else
   {
    int m = pickLoad( mouse_pos );
    if( m > -1 )
    {
     setCursor( Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR) );
    }
    else
    {
     setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
    }
   }
  }
 }

 /*============================  mousePressedWC ========================*/
 /**
  * Handles the event of the user pressing down the mouse button.
  */
 public void mousePressedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  last_x = mouse_pos.x;
  last_y = mouse_pos.y;

  // check to see whether dragging supports or changing a member load
  if( !support_insertion && !support_deletion )
  {
   int n = pickNode( mouse_pos );
   if( n > -1 )
   {
    pressedInNode = true;
    pickedNode = n;
    setCursor( Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR) );
   }
   else
   {
    pressedInNode = false;
    int m = pickLoad( mouse_pos );
    if( m > -1 )
    {
     pressedInLoad = true;
     pickedMemb = (CrossDriver.Member)target.driver.member.elementAt(m);
     setCursor( Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR) );
    }
    else
    {
     pressedInLoad = false;
    }
   }
  }
  else if( support_insertion && !support_deletion )
  {
   int m = getIntSupMember( mouse_pos );
   if( m > -1 )
   {
    if( target.driver.addIntSupport( m, mouse_pos.x ) )
    {
     updateNodePos( m+1, mouse_pos );
     target.driver.resetCrossSolver();
     target.model_canvas.reset();
     target.deform_canvas.reset();
     target.moment_canvas.reset();
     target.solution_canvas.reset();
     if( target.driver.moreCrossSteps() )
      target.setMsg( "Moments are not balanced yet." );
     else
      target.setMsg( "Cross Solution performed." );
    }
   }
   else if( m == -1 )
   {
    target.setMsg( "An internal support cannot be inserted: resulting member length smaller than minimum allowed." );
   }
   else
   {
    target.setMsg( "An internal support could not be inserted outside the beam." );
   }
  }
 }

/*============================  mouseDraggedWC =========================*/
 /**
  * Handles the event of a user dragging the mouse while holding down
  * the mouse button.
  */ 
 public void mouseDraggedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  // check to see whether dragging supports or changing a member load
  if( !support_insertion && !support_deletion )
  {
   if( pressedInNode )
   {
    updateNodePos( pickedNode, mouse_pos );
   }
   else if( pressedInLoad )
   {
    updateLoadValue( pickedMemb, mouse_pos );
   }
  }
  last_x = mouse_pos.x;
  last_y = mouse_pos.y;
 }

/*===========================  mouseReleasedWC =========================*/
 /**
  * Handles the event of a user releasing the mouse button.
  */ 
 public void mouseReleasedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  if( support_insertion )
  {
   target.controls.resetButtons();
  }
  else if( support_deletion )
  {
   int n = pickNode( mouse_pos );
   if( (n < 1) || (n > target.driver.member.size()-1) )
   {
    target.setMsg( "An internal support could not be found at this position." );
   }
   else
   {
    if( target.driver.delIntSupport( n-1 ) )
    {
     target.driver.resetCrossSolver();
     target.model_canvas.reset();
     target.deform_canvas.reset();
     target.moment_canvas.reset();
     target.solution_canvas.reset();
     if( target.driver.moreCrossSteps() )
      target.setMsg( "Moments are not balanced yet." );
     else
      target.setMsg( "Cross Solution performed." );
    }
    else
    {
     target.setMsg( "The minimum number of beam members is 2." );
    }
   }
   target.controls.resetButtons();
  }
  else // check to see whether dragging supports or changing a member load
  {
   if( pressedInNode )
   {
    updateNodePos( pickedNode, mouse_pos );
    target.driver.resetCrossSolver();
    target.model_canvas.reset();
    target.deform_canvas.reset();
    target.moment_canvas.reset();
    target.solution_canvas.reset();
    if( target.driver.moreCrossSteps() )
     target.setMsg( "Moments are not balanced yet." );
    else
     target.setMsg( "Cross Solution performed." );
    setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
   }
   else if( pressedInLoad )
   {
    updateLoadValue( pickedMemb, mouse_pos );
    target.driver.resetCrossSolver();
    target.model_canvas.reset();
    target.deform_canvas.reset();
    target.moment_canvas.reset();
    target.solution_canvas.reset();
    if( target.driver.moreCrossSteps() )
     target.setMsg( "Moments are not balanced yet." );
    else
     target.setMsg( "Cross Solution performed." );
    setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
   }
  }
  pressedInNode = false;
  pressedInLoad = false;
 }
}

/**
 * This class defines the load arrow figure.
 */
 class CrossLoadArrow
 {
  Point2D.Double toppt;
  Point2D.Double botpt;
  Point2D.Double leftpt;
  Point2D.Double rightpt;
  Line2D.Double arrowln;
  Line2D.Double leftln;
  Line2D.Double rightln;	

/*==========================  Constructor  =========================*/
	
  CrossLoadArrow()
  {
   toppt   = new Point2D.Double(  0.0,  1.0 );
   botpt   = new Point2D.Double(  0.0,  0.0 );
   leftpt  = new Point2D.Double( -0.2,  0.2 );
   rightpt = new Point2D.Double(  0.2,  0.2 );
   arrowln = new Line2D.Double( toppt, botpt );
   leftln  = new Line2D.Double( leftpt, botpt );
   rightln = new Line2D.Double( rightpt, botpt );
  }

/*=============================  draw  =============================*/
/**
 * Draw the support figure.
 */
  void draw( Graphics2D g, double x, double y, double size, double step )
  {
   arrowln.setLine( x + toppt.getX()*size,   y + toppt.getY()*size, 
                    x + botpt.getX()*size,   y + botpt.getY()*size );
   leftln.setLine ( x + leftpt.getX()*step,  y + leftpt.getY()*step, 
                    x + botpt.getX()*step,   y + botpt.getY()*step );
   rightln.setLine( x + rightpt.getX()*step, y + rightpt.getY()*step, 
                    x + botpt.getX()*step,   y + botpt.getY()*step );
   g.draw( arrowln );
   g.draw( leftln );
   g.draw( rightln );
  }
 } 
  
  
/**
 * This class defines the side bars of the beam's dimension lines.
 */
 class CrossRuleSB
 {
  Point2D.Double toppt;
  Point2D.Double botpt;
  Point2D.Double rightpt;
  Point2D.Double leftpt;
  Line2D.Double  vertln;
  Line2D.Double  diagln;
  
  /*==========================  Constructor  =========================*/
  CrossRuleSB()
  { 		
   toppt   = new Point2D.Double( 0.00,  0.50 );
   botpt   = new Point2D.Double( 0.00, -0.50 );
   rightpt = new Point2D.Double( 0.50,  0.50 );
   leftpt  = new Point2D.Double(-0.50, -0.50 );
   vertln  = new Line2D.Double( toppt,  botpt  );
   diagln  = new Line2D.Double( leftpt, rightpt);
  }
 		
 /*=============================  draw  =============================*/
 /**
  * Draw the rule figure.
  */
  void draw( Graphics2D g, double x, double y, double size )
  {
   vertln.setLine( x + toppt.getX()*size,   y + toppt.getY()*size, 
                   x + botpt.getX()*size,   y + botpt.getY()*size   );
   diagln.setLine( x + leftpt.getX()*size,  y + leftpt.getY()*size, 
                   x + rightpt.getX()*size, y + rightpt.getY()*size );
  
   g.draw(vertln);
   g.draw(diagln);
  }  
 }