/*
 * @(#)CrossMomentCanvas.java   1.0 31/01/2000
 */
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.text.*;

import wc.*;

/**
 * CrossMomentCanvas.java - Plot the moment diagram for each 
 * interaction of the Cross Process of continous beams.
 *
 * <p>
 * Description:
 * <p>
 * ===============================================================
 * <p>
 * This file contains an implementation of the Cross Process
 * for continuous beams.  For more details of this process,
 * refer to class notes of course CIV1127-Analise de Estruturas
 * II, Dept. of Civil Engineering, PUC-Rio.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 31/01/2000
 * @author     Luiz Fernando Martha
 * @author     Andre Cahn
 *
 */
class CrossMomentCanvas extends CanvasWC
{
 /** Target main interface panel */
 CrossPanel target;

 /** beam line */
 Line2D.Double beam;

 /** first structure's support(free rotation) */
 CrossInitFreeRotSup initfreesup;

 /** first structure's support(fix rotation) */
 CrossInitFixRotSup initfixsup;

 /** internal support */
 CrossIntSup intsup;

 /** last structure's support(free rotation) */
 CrossEndFreeRotSup endfreesup;

 /** last structure's support(fix rotation) */
 CrossEndFixRotSup endfixsup;

 /** Beam total length */
 double len;

 /** general path of diagram lines */
 GeneralPath diagram;

 /** maximum moment line */
 Line2D.Double locmax_line;

 /** stroke object of beam */ 
 BasicStroke beam_stroke;

 /** stroke object of support */
 BasicStroke support_stroke;

 /** stroke object of diagram */
 BasicStroke diagram_stroke;

 /** number format object for no decimal places */
 DecimalFormat form0;

 /** number format object for 1 decimal places */
 DecimalFormat form1;

 /** number format object for 2 decimal places */
 DecimalFormat form2;

 /** Holds the coordinates of the user's last mousePressed event*/
 double last_x, last_y;

/*==========================  Constructor  =========================*/

 CrossMomentCanvas( CrossPanel target )
 {
  super( UPWARD_Y );
  this.target = target;
  initfreesup = new CrossInitFreeRotSup();
  initfixsup = new CrossInitFixRotSup();
  intsup = new CrossIntSup();
  endfreesup = new CrossEndFreeRotSup();
  endfixsup = new CrossEndFixRotSup();
  len = target.driver.getBeamLength();
  beam = new Line2D.Double( 0.0, 0.0, len, 0.0 );
  diagram = new GeneralPath();
  locmax_line = new Line2D.Double( 0.0, 0.0, 0.0, -1.0 );
  beam_stroke = new BasicStroke( (float)(len*0.002) );
  support_stroke = new BasicStroke( (float)(len*0.001) );
  diagram_stroke = new BasicStroke( (float)(len*0.002) );

  setFont( new Font( "helvetica", Font.PLAIN, 10 ) );

  try {
   form0 = new DecimalFormat( "##0" );
   form1 = new DecimalFormat( "##0.0" );
   form2 = new DecimalFormat( "##0.00" );
  } 
  catch( IllegalArgumentException e ) {
  }

  setBackground( Color.white );
  setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );
 }

 private double MomentDiagram( double len, double q,
                               double moml, double momr, double x )
 {
  return( -moml + ((moml + momr)/len)*x + q*len*0.5*x - q*x*x*0.5 );
 }

 private void createDiagram()
 {
  CrossDriver.Member memb;
  double x_beam = 0.0;
  double step = len * 0.02;
  double diagram_fac;
  int    n_member = target.driver.member.size();
  int    n_seg;
  double x_member;
  double m_max = 0.0;
  double m;

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   m  = Math.abs( memb.q * (memb.len*memb.len) / 8.0 );
   if( m > m_max )
    m_max = m;
  }

  diagram_fac = (world_area.height * 0.40) / m_max;

  diagram.reset();
  diagram.moveTo( (float)x_beam, 0.0f );

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   n_seg = (int)(memb.len/step);
   x_member = 0;
   for( int j=0; j<=n_seg; j++ )
   {
    m = MomentDiagram( memb.len, memb.q, 
                       memb.ml, memb.mr, x_member );
    diagram.lineTo( (float)(x_beam+x_member), (float)(-m*diagram_fac) );
    x_member += step;
   }
   m = MomentDiagram( memb.len, memb.q, 
                      memb.ml, memb.mr, memb.len );
   diagram.lineTo( (float)(x_beam+memb.len), (float)(-m*diagram_fac) );
   diagram.lineTo( (float)(x_beam+memb.len), 0.0f );
   x_beam += memb.len;
  }
 }

 private void displaySupports( Graphics2D g )
 {
  CrossDriver.Member memb;
  CrossDriver.Member next_memb;
  double  x = 0.0;
  int     n_member = target.driver.member.size();
  boolean filled;

  g.setStroke( support_stroke );

  // Display initial support
  g.setColor( Color.blue );
  if( target.driver.getInitRotationCode() == 0 )
   initfreesup.draw( g, 0.0, 0.0, len * 0.02 );
  else
   initfixsup.draw( g, 0.0, 0.0, len * 0.02 );

  // Display all internal supports
  for( int i=0; i<n_member-1; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   next_memb = (CrossDriver.Member)target.driver.member.elementAt(i+1);
   if( Math.abs(memb.mr + next_memb.ml) < target.driver.mom_tol )
    filled = false;
   else
    filled = true;
   x += memb.len;
   intsup.draw( g, filled, Color.blue, Color.red, x, 0.0, len * 0.02 );
  }

  // Display end support
  g.setColor( Color.blue );
  if( target.driver.getEndRotationCode() == 0 )
   endfreesup.draw( g, len, 0.0, len * 0.02 );
  else
   endfixsup.draw( g, len, 0.0, len * 0.02 );
 }

 private void displayValues( Graphics2D g )
 {
  DecimalFormat form;
  String val;
  CrossDriver.Member memb;
  double x_beam = 0.0;
  double diagram_fac;
  int    n_member = target.driver.member.size();
  double m_max = 0.0;
  double m;
  double x_locmax;
  double m_locmax;
  double mr_prev = 0.0;

  // Get moment value number format
  switch( target.driver.getMomentPrecision() )
  {
   case 0:
    form = form0;
    break;
   default:
   case 1:
    form = form1;
    break;
   case 2:
    form = form2;
    break;
  }

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   m  = Math.abs( memb.q * (memb.len*memb.len) / 8.0 );
   if( m > m_max )
    m_max = m;
  }

  diagram_fac = (world_area.height * 0.40) / m_max;

  for( int i=0; i<n_member; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   if( memb.ml != 0.0 )
   {
    // Display left moment value
    val = new String( form.format(Math.abs(memb.ml)) );
    // If left moment is equal to previous, center value display
    if( Math.abs(memb.ml + mr_prev) < target.driver.mom_tol )
    {
     if( memb.ml > 0.0 )
     {
      setTextAlignment( CanvasWC.BASE_CENTER );
      drawString( val, x_beam, memb.ml*diagram_fac + len*0.002 );
     }
     else
     {
      setTextAlignment( CanvasWC.NORTH );
      drawString( val, x_beam, memb.ml*diagram_fac - len*0.002 );
     }
    }
    else
    {
     if( memb.ml > 0.0 )
     {
      setTextAlignment( CanvasWC.BASE_LEFT );
      drawString( val, x_beam + len*0.003, memb.ml*diagram_fac + len*0.002 );
     }
     else
     {
      setTextAlignment( CanvasWC.NORTH_WEST );
      drawString( val, x_beam + len*0.003, memb.ml*diagram_fac - len*0.002 );
     }
    }
   }

   // Display local maximum moment value and line
   if( memb.q != 0.0 )
   {
    x_locmax = (memb.len*0.5) + ((memb.ml+memb.mr) / (memb.q*memb.len));
    if( (x_locmax > (len * 0.02)) && (x_locmax < (memb.len - (len * 0.02))) )
    {
     m_locmax = MomentDiagram( memb.len, memb.q, memb.ml, memb.mr, x_locmax );
     val = new String( form.format(Math.abs( m_locmax )) );
     if( m_locmax < 0.0 )
     {
      setTextAlignment( CanvasWC.BASE_CENTER );
      drawString( val, x_beam + x_locmax, -m_locmax*diagram_fac + len*0.002 );
     }
     else
     {
      setTextAlignment( CanvasWC.NORTH );
      drawString( val, x_beam + x_locmax, -m_locmax*diagram_fac - len*0.002 );
     }

     locmax_line.setLine( x_beam + x_locmax, 0.0, 
                          x_beam + x_locmax, -m_locmax*diagram_fac );
     g.draw( locmax_line );
    }
   }

   // Display right moment value
   x_beam += memb.len;
   if( memb.mr != 0.0 )
   {
    boolean vis = true;
    // If right moment is equal to next left moment, do not display value
    if( i < n_member-1 )
    {
     CrossDriver.Member next_memb = 
                (CrossDriver.Member)target.driver.member.elementAt(i+1);
     if( Math.abs(memb.mr + next_memb.ml) < target.driver.mom_tol )
      vis = false;
    }
    if( vis )
    {
     val = new String( form.format(Math.abs(memb.mr)) );
     if( memb.mr < 0.0 )
     {
      setTextAlignment( CanvasWC.BASE_RIGHT );
      drawString( val, x_beam - len*0.001, -memb.mr*diagram_fac + len*0.002 );
     }
     else
     {
      setTextAlignment( CanvasWC.NORTH_EAST );
      drawString( val, x_beam - len*0.001, -memb.mr*diagram_fac - len*0.002 );
     }
    }
   }
   mr_prev = memb.mr;
  }
 }

 void reset()
 {
  len = target.driver.getBeamLength();
  beam.setLine( 0.0, 0.0, len, 0.0 );
  beam_stroke = new BasicStroke( (float)(len*0.002) );
  support_stroke = new BasicStroke( (float)(len*0.001) );
  diagram_stroke = new BasicStroke( (float)(len*0.002) );
  setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );
  repaint();
 }

 public void redraw( Graphics2D g )
 {
  CrossDriver.Member memb;
  double x = 0.0;

  g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                         RenderingHints.VALUE_ANTIALIAS_ON);

  // Update beam moment diagram
  createDiagram();

  // Display beam line
  g.setColor( Color.blue );
  g.setStroke( beam_stroke );
  g.draw( beam );

  // Display all supports
  displaySupports( g );

  // Display moment diagram
  g.setStroke( diagram_stroke );
  g.draw( diagram );

  // Display moment values
  displayValues( g );
 }

 // Handles the event of the user pressing down the mouse button.
 public void mousePressedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  last_x = mouse_pos.x;
  last_y = mouse_pos.y;
 }

 // Handles the event of a user dragging the mouse while holding down the mouse button.
 public void mouseDraggedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }

 // Handles the event of a user releasing the mouse button.
 // Check to see whether an internal node was hit by the mouse.
 // If that is the case, perfom one Cross iteration, balancing the
 // moments at that node.
 public void mouseReleasedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  CrossDriver.Member memb;
  double tol_fac = 0.02;
  double tol = len * tol_fac;
  int    n_member = target.driver.member.size();
  double x_beam = 0.0;
  double y_beam = 0.0;
  double x_member;

  if( ! target.driver.moreCrossSteps() )
  {
   target.setMsg( "Cross Solution performed." );
   return;
  }

  for( int i=0; i<n_member-1; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x_beam += memb.len;

   if( (Math.abs(mouse_pos.x - x_beam) <= tol) &&
       (Math.abs(mouse_pos.y - y_beam) <= tol) )
   {
    if( target.driver.userStepCross( i ) )
    {
     target.deform_canvas.repaint();
     target.moment_canvas.repaint();
     target.solution_canvas.update();
     if( ! target.driver.moreCrossSteps() )
      target.setMsg( "Cross Solution performed." );
     else
      target.setMsg( "Moments are not balanced yet." );
    }
    else
    {
     target.setMsg( "Moments at node are already balanced.  Try another node." );
    }
    return;
   }
  }
 }
}