/*
 * @(#)CrossSolutionCanvas.java   1.0 17/01/2000
 */

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.Dimension.*;
import java.text.*;

import wc.*;
/**
 * CrossSolutionCanvas.java - Shows the results of the
 * Cross Process of continous beams.
 *
 * <p>
 * Description:
 * <p>
  * ===============================================================
 * <p>
 * This file contains an implementation of the Cross Process
 * for continuous beams.  For more details of this process,
 * refer to class notes of course CIV1127-Analise de Estruturas
 * II, Dept. of Civil Engineering, PUC-Rio.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 17/12/1999
 * @author     Luiz Fernando Martha
 * @author     Andre Cahn
 *
 */
class CrossSolutionCanvas extends CanvasWC
{
 /** Target main interface panel */	
 CrossPanel target;

 /** beam line  */
 Line2D.Double beam;                

 /** Beam total length */
 double len;

 /** offset value to shift beam line from top */
 double beam_offset;

 /** first structure's support(free rotation) */
 CrossInitFreeRotSup initfreesup;

 /** first structure's support(fix rotation) */
 CrossInitFixRotSup initfixsup;

 /** internal support */
 CrossIntSup intsup;

 /** last structure's support(free rotation) */
 CrossEndFreeRotSup endfreesup;

 /** last structure's support(fix rotation) */
 CrossEndFixRotSup endfixsup;

 /** carry-over arrow */
 CrossCarryOverArrow carryarrow;

 /** Solution display offset from top (in WC) */
 double y_offset;

 /** Solution window in canvas (in WC) */
 Rectangle2D.Double sol_area;

 /** Solution line size in raster coordinates */
 int    sol_linesize;

 /** Solution line height in WC */
 double h_solline;

 /** Start Y coord of solution (in WC) (y_sol<=y_offset) */
 double y_sol;

 /** Solution display height in WC */
 double h_sol;

 /** If true, shift display of Cross iterations up on the canvas */
 boolean shiftUp;

 /** horizontal line of solution table */
 Line2D.Double hline;

 /** vertical line of solution table */
 Line2D.Double vline;

 /** stroke object of carry-over arrow */
 BasicStroke carry_stroke;

 /** stroke object of beam */
 BasicStroke beam_stroke;

 /** stroke object of support */
 BasicStroke support_stroke;

 /** stroke object of solution line */
 BasicStroke sol_stroke;

 /** number format object for no decimal places */
 DecimalFormat form0;

 /** number format object for 1 decimal places */
 DecimalFormat form1;

 /** number format object for 2 decimal places */
 DecimalFormat form2;

 /** distribution coefficient value number format */
 DecimalFormat form_distcoef; 

 /** Holds the coordinates of the user's last mouse event.*/
 double last_x, last_y;

 /** True if the user pressed, dragged or released the mouse intside the 
   * soluction rectangle; false otherwise.*/
 boolean pressedIn;  

/*==========================  Constructor  =========================*/

 public CrossSolutionCanvas( CrossPanel target )
 {
  super( DOWNWARD_Y );
  this.target = target;
  initfreesup = new CrossInitFreeRotSup();
  initfixsup  = new CrossInitFixRotSup();
  intsup      = new CrossIntSup();
  endfreesup  = new CrossEndFreeRotSup();
  endfixsup   = new CrossEndFixRotSup();
  carryarrow  = new CrossCarryOverArrow();
  len = target.driver.getBeamLength();
  beam_offset = 0.02 * len;

  shiftUp = false;
  pressedIn = false;
  setFont( new Font( "helvetica", Font.PLAIN, 10 ) );
  setBackground( Color.white );
  sol_linesize = 15;  /* in pixels */

  carry_stroke   = new BasicStroke( (float)(len*0.0015) );
  beam_stroke    = new BasicStroke( (float)(len*0.002) );
  support_stroke = new BasicStroke( (float)(len*0.001) );
  sol_stroke     = new BasicStroke( (float)(len*0.001) );

  try {
   form0 = new DecimalFormat( "##0" );
   form1 = new DecimalFormat( "##0.0" );
   form2 = new DecimalFormat( "##0.00" );
   form_distcoef = new DecimalFormat( "####0.00" );
  } 
  catch( IllegalArgumentException e ) {
  }

  beam  = new Line2D.Double( 0.0, 0.0, len, 0.0 );
  hline = new Line2D.Double( 0.0, 0.0, len, 0.0 );
  vline = new Line2D.Double( 0.0, 0.0, 0.0, 1.0 );
 }

/*========================= displaySupports ========================*/
/**
  * This method displays all supports.
  */
 private void displaySupports( Graphics2D g )
 {
  CrossDriver.Member memb;
  double  x = 0.0;       
  int     n_member = target.driver.member.size();
 
  CrossDriver.Member memb_next;      

  g.setStroke( support_stroke );
  g.setColor( Color.blue );

  // Display initial support
  if( target.driver.getInitRotationCode() == 0 )
   initfreesup.draw( g, 0.0, beam_offset, -len * 0.02 );
  else
   initfixsup.draw( g, 0.0, beam_offset, -len * 0.02 );

  // Display all internal supports
  for( int i=0; i<n_member-1; i++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(i);
   x += memb.len;
   intsup.draw( g, x, beam_offset, -len * 0.02 );
  }

  // Display end support
  if( target.driver.getEndRotationCode() == 0 )
   endfreesup.draw( g, len, beam_offset, -len * 0.02 );
  else
   endfixsup.draw( g, len, beam_offset, -len * 0.02 );
 }

/*========================= displaySolution ========================*/
/**
  * This method displays the solution values, carry-over arrows,
  * and the table lines.
  */
 private void displaySolution( Graphics2D g )
 {
  double worldheight;   /* Holds the value of the world height */
  String val;           /* display the number of the interation */  
  int supinit;          /* first node condition (free or fix) */
  int supend;           /* last node condition (free or fix) */
  String mlval;         /* display the initial left moment value */
  String mrval;         /* display the initial right moment value */
  String bmlval;        /* display balancing moment at left of node */
  String bmrval;        /* display balancing moment at right of node */
  String tmlval;        /* display carry-over moment at left of node */
  String tmrval;        /* display carry-over moment at right of node */  
  double y;             /* y coordenate */
  double y_start = -1.0;           /* y coordenate at the begining of a beam */
  double y_end = -1.0;             /* y coordenate at the end of a beam */
  CrossDriver.Member memb;         /*  */
  CrossDriver.Member memb_next;    /*  */
  CrossSolver.BalMoment crossiter; /*  */  
  double  x = 0.0;                 /* x coordenate */
  double  x_prev = 0.0;            /* x coordenate of a anterior beam */
  double  x_next = 0.0;            /* x coordenate of the next beam */
  int     node;                    /* node number */
  DecimalFormat form;              /* moment value number format */
  double  hshift = len*0.003;      /* horizontal shift for text display */
  double  vshift = len*0.002;      /* vertical shift for text display */
  double  arrow_size;              /* carry-over arrow size */
  double  nextarrow_size;          /* next carry-over arrow size */
  int     n_member = target.driver.member.size(); /** number of beams */
  double  mom;                     /* auxiliary moment value */
  double  mom_tol = 0.1;           /* tolerance for moment balancing */
 
  g.setColor( Color.black );

  supinit = target.driver.getInitRotationCode();
  supend = target.driver.getEndRotationCode();
  
  worldheight = world_area.height;
  
  /* horizontal shift for carry over arrow display*/
  double  arrowhshift = len*0.06;
    
  // Get moment value number format
  switch( target.driver.getMomentPrecision() )
  {
   case 0:
    form = form0;
    mom_tol = 1.0;
    break;
   default:
   case 1:
    form = form1;
    mom_tol = 0.1;
    break;
   case 2:
    form = form2;
    mom_tol = 0.01;
    break;
  }

  //Display moment distribution coefficients           
  double kl=0;  // left relative rotational stiffness
  double kr=0;  // right relative rotational stiffness 
  String dlval; // string for values of left distribution coefficient 
  String drval; // string for values of right distribution coefficient 
  // vertical position for distribution coefficient value display
  double dc_pos = beam_offset-0.8*h_solline;

  x=0.0;
  for( int j=0; j<n_member-1; j++ )
  {     
   memb = (CrossDriver.Member)target.driver.member.elementAt(j);
   memb_next =(CrossDriver.Member)target.driver.member.elementAt(j+1);
  
   if(j==0)
   {
    x=memb.len;
   }
     
   if( (j == 0) && (supinit == 0) )
   {
    kl = 3.0 / memb.len;  
   }
   else
   {
    kl = 4.0 / memb.len;  
   }
   if( (j == n_member-2) && (supend == 0) )
   {
    kr = 3.0 / memb_next.len;  
   }
   else
   {
    kr = 4.0 / memb_next.len;  
   }

   dlval = new String( form_distcoef.format( kl/(kl+kr) ) );
   setTextAlignment( CanvasWC.NORTH_EAST );
   drawString( dlval, x - hshift, dc_pos);

   drval = new String( form_distcoef.format( kr/(kl+kr) ) );
   setTextAlignment( CanvasWC.NORTH_WEST );
   drawString( drval, x + (2.5*hshift), dc_pos );

   x += memb_next.len;   
  } 
  
  // First solution line correspond to fix end moments.
  g.setStroke( sol_stroke );  
  y = y_sol + vshift;
  if( y >= y_offset )
  {
   if( y_start < 0.0 )
    y_start = y;
   hline.setLine( 0.0, y, len, y );
   y_end = y;
   g.setStroke( sol_stroke );
   g.draw( hline );

   val = new String( "0" );
   setTextAlignment( CanvasWC.NORTH_EAST );
   drawString( val, - hshift, y + vshift );
   
   // Display initial moment values ( first line solution )
   double xl = 0.0;
   double xr = 0.0;

   for(int i=0; i<n_member; i++)
   {    
    memb = (CrossDriver.Member)target.driver.member.elementAt(i);
    
    xr = xl + memb.len;
    
    if(i==0)
    {   		
     if( supinit == 0 )
     {      
      if( mom_tol == 1.0 )
       mlval = new String( "+0" );
      else
       mlval = new String( "+" + form.format( 0.0 ) );
      mom = - memb.q * (memb.len*memb.len)/8.0;
      if( Math.abs(mom) < mom_tol )
       mom = 0.0;
      if( mom >= 0.0 )
       mrval = new String( "+" + form.format( mom ) );
      else
       mrval = new String( form.format( mom ) );
     }
     else
     {
      mom = memb.q * (memb.len*memb.len)/12.0;
      if( Math.abs(mom) < mom_tol )
       mom = 0.0;
      if( mom >= 0.0 )
       mlval = new String( "+" + form.format( mom ) );
      else
       mlval = new String( form.format( mom ) );
      mom = - memb.q * (memb.len*memb.len)/12.0;
      if( Math.abs(mom) < mom_tol )
       mom = 0.0;
      if( mom >= 0.0 )
       mrval = new String( "+" + form.format( mom ) );
      else
       mrval = new String( form.format( mom ) );
     }      
    }
    else if(i==n_member-1)
    {
     if( supend == 0 )
     {
      mom = memb.q * (memb.len*memb.len)/8.0;
      if( Math.abs(mom) < mom_tol )
       mom = 0.0;
      if( mom >= 0.0 )
       mlval = new String( "+" + form.format( mom ) );
      else
       mlval = new String( form.format( mom ) );
      if( mom_tol == 1.0 )
       mrval = new String( "+0" );
      else
       mrval = new String( "+" + form.format( 0.0 ) );
     }
     else
     {
      mom = memb.q * (memb.len*memb.len)/12.0;
      if( Math.abs(mom) < mom_tol )
       mom = 0.0;
      if( mom >= 0.0 )
       mlval = new String( "+" + form.format( mom ) );
      else
       mlval = new String( form.format( mom ) );
      mom = - memb.q * (memb.len*memb.len)/12.0;
      if( Math.abs(mom) < mom_tol )
       mom = 0.0;
      if( mom >= 0.0 )
       mrval = new String( "+" + form.format( mom ) );
      else
       mrval = new String( form.format( mom ) );
     }
    }
    else
    {   
     mom = memb.q * (memb.len*memb.len)/12.0;
     if( Math.abs(mom) < mom_tol )
      mom = 0.0;
     if( mom >= 0.0 )
      mlval = new String( "+" + form.format( mom ) );
     else
      mlval = new String( form.format( mom ) );
     mom = - memb.q * (memb.len*memb.len)/12.0;
     if( Math.abs(mom) < mom_tol )
      mom = 0.0;
     if( mom >= 0.0 )
      mrval = new String( "+" + form.format( mom ) );
     else
      mrval = new String( form.format( mom ) );
    }  

    setTextAlignment( CanvasWC.NORTH_WEST  );
    drawString( mlval, xl + (2.4*hshift), y + vshift);
    setTextAlignment( CanvasWC.NORTH_EAST );
    drawString( mrval, xr - hshift, y + vshift);

    xl = xr;
   }
  }

  // The remaining number of solution lines is equal to 
  // number of moment balancing operations performed by 
  // Cross solver.
  int n_sollines = target.driver.solver.balmom.size();  
  for( int i=0; i<n_sollines; i++ )
  {
   g.setStroke( sol_stroke );
   crossiter = (CrossSolver.BalMoment)target.driver.solver.balmom.elementAt(i);
  
   y = y_sol + (i+1)*h_solline + vshift;
   if( (y >= y_offset) && (y < world_area.height) )
   {
    if( y_start < 0.0 )
     y_start = y;
    hline.setLine( 0.0, y, len, y );
    y_end = y;
    g.draw( hline );

    // If next horizontal line will be outside canvas, quit displaying values.
    if( (y + h_solline + vshift) > world_area.height )
     break;

    val = new String( "" + (i+1) );
    setTextAlignment( CanvasWC.NORTH_EAST );
    drawString( val, - hshift, y + vshift );

    // Find the position of the node of this Cross iteration
    x = 0.0;
    for( int j=0; j<n_member-1; j++ )
    {
     x_prev = x;
     memb = (CrossDriver.Member)target.driver.member.elementAt(j);
     x += memb.len;
     memb_next = (CrossDriver.Member)target.driver.member.elementAt(j+1);
     x_next = x + memb_next.len;
     if( crossiter.n == j )
      break;
    }
        
    // Display solution values
    crossiter = (CrossSolver.BalMoment)target.driver.solver.balmom.elementAt(i);  
    
    node = crossiter.n;

    if( Math.abs(crossiter.bml) < mom_tol )
     crossiter.bml = 0.0;
    if( crossiter.bml >= 0 )
     bmlval = new String( "+" + form.format(crossiter.bml) );
    else
     bmlval = new String( form.format(crossiter.bml) );

    if( Math.abs(crossiter.bmr) < mom_tol )
     crossiter.bmr = 0.0;
    if( crossiter.bmr >= 0 )
     bmrval = new String( "+" + form.format(crossiter.bmr) );     
    else
     bmrval = new String( form.format(crossiter.bmr) );     

    if( Math.abs(crossiter.tml) < mom_tol )
     crossiter.tml = 0.0;
    if( crossiter.tml >= 0 )
     tmlval = new String( "+" + form.format(crossiter.tml) );   
    else
     tmlval = new String( form.format(crossiter.tml) );   

    if( Math.abs(crossiter.tmr) < mom_tol )
     crossiter.tmr = 0.0;
    if( crossiter.tmr >= 0 )
     tmrval = new String( "+" + form.format(crossiter.tmr) );
    else
     tmrval = new String( form.format(crossiter.tmr) );

    setTextAlignment( CanvasWC.NORTH_EAST );
    drawString( bmlval, x - hshift, y + vshift);
    setTextAlignment( CanvasWC.NORTH_WEST );
    drawString( bmrval, x + (2.4*hshift), y + vshift);
    setTextAlignment( CanvasWC.NORTH_WEST );
    drawString( tmlval, x_prev + (2.4*hshift), y + vshift);
    setTextAlignment( CanvasWC.NORTH_EAST );
    drawString( tmrval, x_next - hshift, y + vshift); 

  
    // Display carry-over arrows    
    g.setStroke( carry_stroke );
    memb = (CrossDriver.Member)target.driver.member.elementAt(node);
    memb_next = (CrossDriver.Member)target.driver.member.elementAt(node+1);
    arrow_size = 0.6*memb.len;
    nextarrow_size = 0.6*memb_next.len;
    carryarrow.draw( g, 0.5*(x_prev+x-0.5*arrow_size),y+h_solline*0.5,arrow_size );	  
    carryarrow.draw( g,0.5*(x_next+x+0.5*nextarrow_size), y+h_solline*0.5,-nextarrow_size);
   }
  }
  g.setStroke( sol_stroke );  
  y = y_sol + h_sol + vshift;
  if( y <= world_area.height )
  {
   hline.setLine( 0.0, y, len, y );
   y_end = y;
   g.draw( hline );
  }
  
  // Display solution table vertical lines
  vline.setLine( 0.0, y_start, 0.0, y_end );
  g.draw( vline );
  x = 0.0;
  for( int j=0; j<n_member-1; j++ )
  {
   memb = (CrossDriver.Member)target.driver.member.elementAt(j);
   x += memb.len;
   vline.setLine( x, y_start, x, y_end );
   g.draw( vline );
  }
  vline.setLine( len, y_start, len, y_end );
  g.draw( vline );
 }

/**
 * This private method updates the solution rectangle height.
 * The new height is proportional to the number of Cross solution
 * lines. If the current value of shiftUp is true, the method
 * checks to see whether the last solution line is bellow the
 * bottom of solution canvas window and, if that is the case, 
 * shifts solution rectangle up such that rectangle bottom be
 * visible. At the end it resets shiftUp to false.
 */
 private void updateSolutionHeight()
 {
  if( sol_area == null )
   return;

  // Compute solution line height based on raster size
  h_solline = (world_area.height * (double)sol_linesize) / (double)raster_area.height;

  // Number of solution lines is equal to number of moment balancing operations
  // performed by Cross solver.
  int n_sollines = target.driver.solver.balmom.size();

  // Set height of solution rectangle based on number of solution lines
  h_sol = (n_sollines+1) * h_solline;

  // Update position solution rectangle if required: shifts 
  // rectangle up such that rectangle bottom is always visible,
  // and resets shiftUp to false.
  if( shiftUp )
  {
   if( (y_sol + h_sol) > world_area.height )
    y_sol = world_area.height - h_sol - len*0.005;
   shiftUp = false;
  }
 }

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

/*===============================  reset  ==============================*/
/**
  * Reset method.
  */
 void reset()
 {
  len = target.driver.getBeamLength();
  beam_offset = 0.02 * len;
  beam_stroke    = new BasicStroke( (float)(len*0.002) );
  support_stroke = new BasicStroke( (float)(len*0.001) );
  sol_stroke     = new BasicStroke( (float)(len*0.001) );

  // Set WC window such that ymin = 0.0 at the top
  setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );
  setWorld( world_area.x, world_area.x + world_area.width, 
            0.0, world_area.height );

  beam.setLine( 0.0, beam_offset, len, beam_offset );

  // Reset solution offset as a fraction of beam size (leave room to draw beam)
  // and set solution window area
  y_offset = 0.05 * len;
  if( sol_area == null )
   sol_area = new Rectangle2D.Double( 0.0, y_offset, len, world_area.height - y_offset );
  else
   sol_area.setRect( 0.0, y_offset, len, world_area.height - y_offset );

  // Reset initial solution position and size
  y_sol = y_offset;

  shiftUp = false;
  repaint();
 }

/*===============================  update ==============================*/
/**
 * This method updates the Cross iteration display.
 * It sets shiftUp as true and redisplay the solution canvas.
 */
 void update()
 {
  shiftUp = true;
  repaint();
 }


/*===============================  redraw  =============================*/
 /**
  * Redisplay method.
  * This method is called every time the canvas is updated.
  * Client's concrete class must implement it.
  * @param g is the 2D graphics context that the client uses to display
  */
 public void redraw( Graphics2D g )
 {
  if( sol_area == null )
  {
   // We can only setup the solution window area after canvas is setup 
   // completely, which occurs after first update (first time redraw is called).
   reset();
   return;
  }
  else
  {
   // Set WC window such that ymin = 0.0 at the top
   setWorld( -len/20.0, len+(len/20.0), -len/20.0, len/20.0 );
   setWorld( world_area.x, world_area.x + world_area.width, 
             0.0, world_area.height );
   sol_area.setRect( 0.0, y_offset, len, world_area.height - y_offset );
  }

  g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                      RenderingHints.VALUE_ANTIALIAS_ON );

  updateSolutionHeight();

  // Display beam line
  g.setColor( Color.blue );
  g.setStroke( beam_stroke );
  g.draw( beam );

  // Display all supports
  displaySupports( g );

  // Draws the newly positioned solution.
  g.setColor( Color.gray );
  g.setStroke( sol_stroke );
  displaySolution( g );
 }

/*=========================== updateSolPosition ========================*/
/**
 * This private methods tries to shift the display of the 
 * Cross iterative solution as a result of a mouse dragging event.
 */
 private void updateSolPosition( Point2D.Double mouse_pos )
 {
  double y_var = mouse_pos.y - last_y;
  double y_solnew = y_sol + y_var;

  // Do not allow shifting the solution top line bellow
  // the top of the solution area.
  if( y_solnew > y_offset )
   return;

  // Do not allow shifting the solution bottom line above
  // the bottom of the solution area.
  if( (y_solnew + h_sol) < (world_area.height - len*0.005) )
   return;

  // Update solution position and redisplay.
  y_sol = y_solnew;
  repaint();
 }

/*==============================  mouseMoved ===========================*/
 /**
  * Handles the event of mouse motion in canvas with no
  * button pressed.
  */
 public void mouseMoved( MouseEvent e )
 {
  if( (raster_area == null) || (sol_area == null) )
   return;

 // Checks whether or not the cursor is inside of the solution area.
 // If that is the case, change cursor feedback to notify the user
 // that he or she might drag the solution area.
  Point2D.Double mouse_pos = raster2world( e.getX(), e.getY() );
  if( sol_area.contains(mouse_pos.x, mouse_pos.y) )
  {
   setCursor( Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR) );
  }
  else
  {
   setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
  }
 }

/*==============================  mousePressedWC ===========================*/
 /**
  * Handles the event of the user pressing down the mouse button.
  */
 public void mousePressedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 // Checks whether or not the cursor is inside of the solution area
 // while the user is pressing the mouse. If that is the case, 
 // update solution rectangle position.
  if( sol_area.contains(mouse_pos.x, mouse_pos.y) )
  {
   last_x = mouse_pos.x;
   last_y = mouse_pos.y;
   pressedIn = true;
   setCursor( Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR) );
  }
  else
  {
   pressedIn = false;
  }
 }

/*==============================  mouseDraggedWC ===========================*/
 /**
  * Handles the event of a user dragging the mouse 
  * while holding down the mouse button.
  */
 public void mouseDraggedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  if( pressedIn )
  {
   updateSolPosition( mouse_pos );
   last_x = mouse_pos.x;
   last_y = mouse_pos.y;
  }  
 }

/*==============================  mouseReleasedWC ===========================*/
 /**
  * Handles the event of a user releasing the mouse button.
  */
 public void mouseReleasedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
  if( pressedIn )
  {
   updateSolPosition( mouse_pos );
  }
  pressedIn = false;
  setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
 }

/**
 * This class defines the carry-over arrow figure.
 */
 class CrossCarryOverArrow
 {    
  Point2D.Double uppt;   /** point of the carry-over arrow */
  Point2D.Double downpt;   /** point of the carry-over arrow */
  Point2D.Double leftpt;  /** point of the carry-over arrow */
  Point2D.Double rightpt; /** point of the carry-over arrow */
  Line2D.Double arrowln;  /** line of the carry-over arrow */
  Line2D.Double upln;   /** line of the carry-over arrow */
  Line2D.Double downln;  /** line of the carry-over arrow */
  
/*==========================  Constructor  =========================*/
  CrossCarryOverArrow()
  {    
   uppt    = new Point2D.Double(  0.2, -0.2 );   
   downpt  = new Point2D.Double(  0.2,  0.2 );
   leftpt  = new Point2D.Double(  0.0,  0.0 );
   rightpt = new Point2D.Double(  1.0,  0.0 );
   arrowln = new Line2D.Double( leftpt, rightpt );
   upln    = new Line2D.Double( leftpt, uppt );
   downln  = new Line2D.Double( leftpt, downpt );   
  }

/*=============================  draw  =============================*/
/**
 * Draw the rotation gauge.
 */
  void draw( Graphics2D g, double x,  double y, double size )
  {
   arrowln.setLine(x + leftpt.getX()*size*0.1,  y + leftpt.getY()*size*0.1, 
                   x + rightpt.getX()*size*0.45, y + rightpt.getY()*size*0.45 );

   upln.setLine(x + leftpt.getX()*size*0.1, y + leftpt.getY()*size*0.1, 
                x + uppt.getX()*size*0.1,   y + uppt.getY()*size*0.1 );

   downln.setLine(x + leftpt.getX()*size*0.1, y + leftpt.getY()*size*0.1, 
                  x + downpt.getX()*size*0.1, y + downpt.getY()*size*0.1 );
   
   g.draw( arrowln );
   g.draw( upln );
   g.draw( downln );
  }
 } 
}